'use client';

import { createContext, useCallback, useContext, useState } from 'react';

import { useOss, useOssInvalidate, useOssUpdate } from '@/backend/oss/useOSS';
import { ErrorData } from '@/components/info/InfoError';
import { LibraryItemID } from '@/models/library';
import { IOperationSchema, IOperationSchemaData } from '@/models/oss';
import { contextOutsideScope } from '@/utils/labels';

interface IGlobalOssContext {
  schema: IOperationSchema | undefined;
  setID: (id: LibraryItemID | undefined) => void;
  setData: (data: IOperationSchemaData) => void;
  loading: boolean;
  loadingError: ErrorData;

  invalidate: () => Promise<void>;
  invalidateItem: (target: LibraryItemID) => void;
  partialUpdate: (data: Partial<IOperationSchema>) => void;
}

const GlobalOssContext = createContext<IGlobalOssContext | null>(null);
export const useGlobalOss = (): IGlobalOssContext => {
  const context = useContext(GlobalOssContext);
  if (context === null) {
    throw new Error(contextOutsideScope('useGlobalOss', 'GlobalOssState'));
  }
  return context;
};

export const GlobalOssState = ({ children }: React.PropsWithChildren) => {
  const [ossID, setID] = useState<LibraryItemID | undefined>(undefined);
  const { schema: schema, error: loadingError, isLoading: loading } = useOss({ itemID: ossID });
  const { update, partialUpdate } = useOssUpdate({ itemID: ossID });
  const { invalidate } = useOssInvalidate({ itemID: ossID });

  const invalidateItem = useCallback(
    (target: LibraryItemID) => {
      if (schema?.schemas.includes(target)) {
        invalidate().catch(console.error);
      }
    },
    [invalidate, schema]
  );

  return (
    <GlobalOssContext
      value={{
        schema,
        setID,
        setData: update,
        loading,
        loadingError,
        partialUpdate,
        invalidateItem,
        invalidate
      }}
    >
      {children}
    </GlobalOssContext>
  );
};
