'use client';

import { useCallback, useState } from 'react';

import { DataCallback } from '@/backend/apiTransport';
import { ICheckConstituentaDTO } from '@/backend/rsform/api';
import { useCheckConstituenta } from '@/backend/rsform/useCheckConstituenta';
import { IConstituenta, type IRSForm } from '@/models/rsform';
import { IExpressionParse } from '@/models/rslang';

function useRSParse({ schema }: { schema?: IRSForm }) {
  const [parseData, setParseData] = useState<IExpressionParse | undefined>(undefined);

  const resetParse = useCallback(() => setParseData(undefined), []);
  const { checkConstituenta: checkInternal, isPending, error, reset } = useCheckConstituenta();

  function checkConstituenta(expression: string, activeCst: IConstituenta, onSuccess?: DataCallback<IExpressionParse>) {
    const data: ICheckConstituentaDTO = {
      definition_formal: expression,
      alias: activeCst.alias,
      cst_type: activeCst.cst_type
    };
    checkInternal({ itemID: schema!.id, data }, parse => {
      setParseData(parse);
      onSuccess?.(parse);
    });
  }

  return { parseData, checkConstituenta, resetParse, error, reset, isPending };
}

export default useRSParse;
